<?php
// $Id: page_theme.admin.inc,v 1.1.2.5 2009/07/24 11:08:11 flanker Exp $
// $Name: DRUPAL-6--1-0-RC2 $

/**
 * @file
 *   Administrative page callbacks for the "Page theme" module.
 */


/**
 * Menu callback; lists all defined themes.
 */
function page_theme_admin_list() {

  $themes = array();
  $result = db_query('SELECT * FROM {page_theme} ORDER BY weight, theme');
  while ($page_theme = db_fetch_object($result)) {

    $row = array();
    $row['name'] = array(
      '#value' => page_theme_get_themes($page_theme->theme),
    );
    $row['pages'] = array(
      '#value' => str_replace("\n", '<br />', check_plain($page_theme->pages)),
    );
    $row['status'] = array(
      '#type' => 'checkbox',
      '#default_value' => $page_theme->status,
    );
    $row['weight'] = array(
      '#type' => 'weight',
      '#default_value' => $page_theme->weight,
      '#delta' => 50,
    );
    $row['edit'] = array(
      '#value' => l(t('edit'), 'admin/build/page-theme/edit/'. $page_theme->theme),
    );
    $row['delete'] = array(
      '#value' => l(t('delete'), 'admin/build/page-theme/delete/'. $page_theme->theme),
    );

    $themes[$page_theme->theme] = $row;
  }

  $form = array();
  $form['themes'] = array(
    '#tree' => TRUE,
  );
  $form['themes'] += $themes;
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  $form['buttons']['cancel'] = array(
    '#type' => 'markup',
    '#value' => l(t('Cancel'), 'admin/build/page-theme'),
  );

  return $form;
}

function page_theme_admin_list_submit($form, &$form_state) {

  foreach ($form_state['values']['themes'] as $theme => $value) {
    db_query('UPDATE {page_theme} SET status = %d, weight = %d WHERE theme = "%s"', $value['status'], $value['weight'], $theme);
  }

  drupal_set_message(t('The configuration has been saved.'));
}

function theme_page_theme_admin_list($form) {

  drupal_add_tabledrag('page-theme-order', 'order', 'sibling', 'page-theme-order-weight');

  $header = array(t('Theme'), t('Pages'), t('Enabled'), t('Weight'), array('data' => t('Operations'), 'colspan' => '2'));
  $rows = array();
  foreach (element_children($form['themes']) as $theme) {
    $form['themes'][$theme]['weight']['#attributes']['class'] = 'page-theme-order-weight';

    $row = array();
    $row[] = drupal_render($form['themes'][$theme]['name']);
    $row[] = drupal_render($form['themes'][$theme]['pages']);
    $row[] = drupal_render($form['themes'][$theme]['status']);
    $row[] = drupal_render($form['themes'][$theme]['weight']);
    $row[] = drupal_render($form['themes'][$theme]['edit']);
    $row[] = drupal_render($form['themes'][$theme]['delete']);

    $rows[] = array('data' => $row, 'class' => 'draggable');
  }
  if (empty($rows)) {
    $rows[] = array(array('data' => t('No themes defined.'), 'colspan' => 6));

    unset($form['buttons']);
  }

  $output = theme('table', $header, $rows, array('id' => 'page-theme-order'));
  $output .= drupal_render($form);

  return $output;
}


/**
 * Menu callback; adds a theme.
 */
function page_theme_admin_add() {

  $form = array();
  $form['theme'] = array(
    '#type' => 'select',
    '#title' => t('Theme'),
    '#description' => t('Choose which theme the page should display in.'),
    '#default_value' => '0',
    '#options' => page_theme_get_theme_options(),
    '#required' => TRUE,
  );
  $form['pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Pages'),
    '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>')),
    '#default_value' => '',
    '#required' => TRUE,
  );
  $form['status'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Status'),
    '#default_value' => array('enabled'),
    '#options' => array(
      'enabled' => t('Enabled'),
    ),
  );
  $form['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => 0,
    '#delta' => 50,
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add theme'),
  );
  $form['buttons']['cancel'] = array(
    '#type' => 'markup',
    '#value' => l(t('Cancel'), 'admin/build/page-theme'),
  );

  return $form;
}

function page_theme_admin_add_validate($form, &$form_state) {

  if (empty($form_state['values']['theme'])) {
    form_set_error('theme', t('Please select a theme.'));
  }
}

function page_theme_admin_add_submit($form, &$form_state) {

  $theme = $form_state['values']['theme'];
  $pages = trim($form_state['values']['pages']);
  $status = is_string($form_state['values']['status']['enabled']);
  $weight = $form_state['values']['weight'];
  db_query('INSERT INTO {page_theme} (theme, pages, status, weight) VALUES ("%s", "%s", %d, %d)', $theme, $pages, $status, $weight);

  drupal_set_message(t('The theme has been added.'));
  $form_state['redirect'] = 'admin/build/page-theme';
}


/**
 * Menu callback; edits a theme.
 */
function page_theme_admin_edit($form_state, $theme = NULL) {

  if (is_null($theme) || !page_theme_theme_exists($theme)) {
    drupal_goto('admin/build/page-theme');
  }

  $page_theme = db_fetch_object(db_query('SELECT * FROM {page_theme} WHERE theme = "%s"', $theme));

  $form = array();
  $form['theme'] = array(
    '#type' => 'value',
    '#value' => $page_theme->theme,
  );
  $form['theme_name'] = array(
    '#type' => 'item',
    '#title' => t('Theme'),
    '#value' => page_theme_get_themes($page_theme->theme),
  );
  $form['pages'] = array(
    '#type' => 'textarea',
    '#title' => t('Pages'),
    '#description' => t("Enter one page per line as Drupal paths. The '*' character is a wildcard. Example paths are %blog for the blog page and %blog-wildcard for every personal blog. %front is the front page.", array('%blog' => 'blog', '%blog-wildcard' => 'blog/*', '%front' => '<front>')),
    '#default_value' => $page_theme->pages,
    '#required' => TRUE,
  );
  $form['status'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Status'),
    '#default_value' => $page_theme->status ? array('enabled') : array(),
    '#options' => array(
      'enabled' => t('Enabled'),
    ),
  );
  $form['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $page_theme->weight,
    '#delta' => 50,
  );
  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Update theme'),
  );
  $form['buttons']['cancel'] = array(
    '#type' => 'markup',
    '#value' => l(t('Cancel'), 'admin/build/page-theme'),
  );

  return $form;
}

function page_theme_admin_edit_submit($form, &$form_state) {

  $theme = $form_state['values']['theme'];
  $pages = trim($form_state['values']['pages']);
  $status = is_string($form_state['values']['status']['enabled']);
  $weight = $form_state['values']['weight'];
  db_query('UPDATE {page_theme} SET pages = "%s", status = %d, weight = %d WHERE theme = "%s"', $pages, $status, $weight, $theme);

  drupal_set_message(t('The theme has been updated.'));
  $form_state['redirect'] = 'admin/build/page-theme';
}


/**
 * Menu callback; deletes a theme.
 */
function page_theme_admin_delete($form_state, $theme = NULL) {

  if (is_null($theme) || !page_theme_theme_exists($theme)) {
    drupal_goto('admin/build/page-theme');
  }

  $form = array();
  $form['theme'] = array(
    '#type' => 'value',
    '#value' => $theme,
  );
  $question = t('Are you sure you want to delete the theme %theme?', array('%theme' => page_theme_get_themes($theme)));
  $path = isset($_GET['destination']) ? $_GET['destination'] : 'admin/build/page-theme';

  return confirm_form($form, $question, $path);
}

function page_theme_admin_delete_submit($form, &$form_state) {

  if ($form_state['values']['confirm']) {
    $theme = $form_state['values']['theme'];
    db_query('DELETE FROM {page_theme} WHERE theme = "%s"', $theme);

    drupal_set_message(t('The theme has been deleted.'));
    $form_state['redirect'] = 'admin/build/page-theme';
    return;
  }
}


/**
 *  Helper functions.
 */
function page_theme_get_theme_options() {
  static $options = array();

  if (!$options) {
    $themes = page_theme_get_themes();

    $result = db_query('SELECT theme FROM {page_theme}');
    while ($page_theme = db_fetch_object($result)) {
      unset($themes[$page_theme->theme]);
    }

    $options[0] = '- '. t('Select theme') .' -';
    $options += $themes;
  }

  return $options;
}

function page_theme_get_themes($theme = NULL) {
  static $themes = array();

  if (!$themes) {
    $themes_data = system_theme_data();
    uasort($themes_data, 'page_theme_sort_themes');

    foreach ($themes_data as $theme_data) {
      if (isset($theme_data->info['core']) && $theme_data->info['core'] == DRUPAL_CORE_COMPATIBILITY && version_compare($theme_data->info['php'], phpversion(), '<=')) {
        $themes[$theme_data->name] = $theme_data->info['name'];
      }
    }
  }

  return $theme ? $themes[$theme] : $themes;
}

function page_theme_sort_themes($a, $b) {
  return strcasecmp($a->info['name'], $b->info['name']);
}

function page_theme_theme_exists($theme) {
  return db_result(db_query('SELECT theme FROM {page_theme} WHERE theme = "%s"', $theme)) !== FALSE;
}
